#!/bin/sh
#
# Prepare archive with test fixtures for tree-sitter unit-tests
#
# The upstream CI is using `master` to fetch the grammars; see `script/fetch-fixtures`.
# Replace with the exact commits or tags to make reproducible.
# Make sure to check the licenses for all the projects we download here.
set -e

DESTDIR="$(dirname "$0" | xargs readlink -f)"
VERSION=$(rpmspec --query --qf '%{version}' --srpm "$DESTDIR/rust-tree-sitter-cli.spec")
MODE=fetch

while [ "$#" -gt 0 ]; do
    case $1 in
        [0-9]*)
            VERSION="$1";;
        --update)
            MODE=update;;
        *)
            break;;
    esac
    shift
done

if [ -z "$VERSION" ]; then
    echo "Cannot determine the tree-sitter-cli version"
    exit 1
fi

OUTFILE="tree-sitter-fixtures-$VERSION.tar.xz"
# WORKDIR="$(mktemp -d tree-sitter-XXXX)"
WORKDIR="${PWD}/tree-sitter-temp-$VERSION"

extract() {
    test -d "$2" || mkdir -p "$2";
    tar xaf "$1" -C "$2" --strip-components=1
}

get () { # args: github_repo, commit, destination
    FILENAME="$(basename "$1")-$2.tar.gz"
    OUT_DIR=${3:-$(basename "$1")-$2}
    echo "Fetching $1@$2 to $OUT_DIR"
    if [ ! -f "$WORKDIR/$FILENAME" ]; then
        curl -LSs  -o "$WORKDIR/$FILENAME" "https://github.com/$1/archive/$2/$FILENAME"
    fi
    rm -rf "$OUT_DIR"
    extract "$WORKDIR/$FILENAME" "$OUT_DIR"
}

# Get date of the commit (tag, sha or any other ref)
get_commit_date () { # args: github_repo, ref
    curl -LSs -H "Accept: application/vnd.github.v3+json" \
            "https://api.github.com/repos/$1/commits/$2" |
        jq -r '.commit.committer.date'
}

# Get last commit in the branch made before the specified timestamp
get_commit_by_date () { # args: github_repo, timestamp, ref (optional)
    curl -LSs -H "Accept: application/vnd.github.v3+json" \
            "https://api.github.com/repos/$1/commits?per_page=1&until=$2${3:+&sha=$3}" |
        jq -r '.[0].sha'
}

# Get test repository commits at the timestamp of the tree-sitter release tag
# (i.e. the exact test data the release tag was tested on)
#
# GitHub API is rate limted, so don't abuse this mode
if [ "$MODE" = "update" ]; then
    TS_TAG_DATE=$(get_commit_date tree-sitter/tree-sitter "v$VERSION")
    echo "# tree-sitter tag date: $TS_TAG_DATE"
    echo "#             LANGUAGE          BRANCH                      CACHED REVISION"

    fetch_grammar () { # args: language, branch, cached revision
        COMMIT=$(get_commit_by_date "tree-sitter/tree-sitter-$1" "$TS_TAG_DATE" "$2")
        printf "fetch_grammar %-17s %-27s %s\n" "$1" "$2" "$COMMIT"
    }
else
    mkdir -p "$WORKDIR"
    cd "$WORKDIR"

    # License: MIT
    get tree-sitter/tree-sitter "v$VERSION" "tree-sitter-$VERSION"
    cd "tree-sitter-$VERSION"

    fetch_grammar () { # args: language, branch, cached revision
        get "tree-sitter/tree-sitter-$1" "${3:-${2:-master}}" "test/fixtures/grammars/$1"
    }
fi

# Verified that all the repositories referenced below are licensed under MIT
# and provide LICENSE files - 2022-09-04
# tree-sitter tag date: 2022-09-02T22:00:47Z
#             LANGUAGE          BRANCH                      CACHED REVISION
fetch_grammar bash              master                      275effdfc0edce774acf7d481f9ea195c6c403cd
fetch_grammar c                 master                      7175a6dd5fc1cee660dce6fe23f6043d75af424a
fetch_grammar cpp               master                      f40125503642845492d87fa56ece3ed26a4ef4db
fetch_grammar embedded-template master                      1a538da253d73f896b9f6c0c7d79cda58791ac5c
fetch_grammar go                master                      aeb2f33b366fd78d5789ff104956ce23508b85db
fetch_grammar html              master                      29f53d8f4f2335e61bf6418ab8958dac3282077a
fetch_grammar java              master                      443275301e60538a1c6f7979ac20a1fda9792934
fetch_grammar javascript        partial-order-precedences   4c2f62903210181ededf461aaa82ca1f804e13e4
fetch_grammar jsdoc             master                      189a6a4829beb9cdbe837260653b4a3dfb0cc3db
fetch_grammar json              master                      11e2cc12d9b267766fb11a06e52952792fd8e3f0
fetch_grammar php               master                      670d1eb6822d8c7ade1c71232e0bef42757b9da7
fetch_grammar python            master                      de221eccf9a221f5b85474a553474a69b4b5784d
fetch_grammar ruby              master                      656abef0645caea793e33c1c773570722463e1d8
fetch_grammar rust              master                      47b061c1e1ba3a7e9c2f450363a50e87de3f7c61
fetch_grammar typescript        master                      082da44a5263599186dadafd2c974c19f3a73d28

if [ "$MODE" != "update" ]; then
    echo "Writing source archive $DESTDIR/$OUTFILE"
    # Drop the examples from the grammar repos. Don't want to check the licenses for unused files
    tar caf "$DESTDIR/$OUTFILE" --exclude=examples -v LICENSE test/fixtures
fi
